@@ -2,10 +2,6 @@
 ;;; Copyright (C) 2016 Jan Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright (C) 2017 Christopher Allan Webber <cwebber@dustycloud.org>
 ;;;
-;;; Also borrowing code from:
-;;; guile-sdl2 --- FFI bindings for SDL2
-;;; Copyright © 2015 David Thompson <davet@gnu.org>
-;;;
 ;;; This file is part of 8sync.
 ;;; However, unlike most of 8sync, which is under the LGPLv3+, this
 ;;; file in particular is licensed under GPLv3+.
@@ -59,27 +55,12 @@
 
 (define %source-dir (dirname (current-filename)))
 
-(define git-file?
-  (let* ((pipe (with-directory-excursion %source-dir
-                 (open-pipe* OPEN_READ "git" "ls-files")))
-         (files (let loop ((lines '()))
-                  (match (read-line pipe)
-                    ((? eof-object?)
-                     (reverse lines))
-                    (line
-                     (loop (cons line lines))))))
-         (status (close-pipe pipe)))
-    (lambda (file stat)
-      (match (stat:type stat)
-        ('directory #t)
-        ((or 'regular 'symlink)
-         (any (cut string-suffix? <> file) files))
-        (_ #f)))))
-
 (package
   (name "guile-8sync")
   (version "git")
-  (source (local-file %source-dir #:recursive? #t #:select? git-file?))
+  (source (local-file %source-dir
+                      #:recursive? #t
+                      #:select? (git-predicate %source-dir)))
   (build-system gnu-build-system)
   (native-inputs `(("autoconf" ,autoconf)
                    ("automake" ,automake)
